// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_DATA_DECODER_PUBLIC_MOJOM_BUNDLED_EXCHANGES_PARSER_MOJOM_SHARED_H_
#define SERVICES_DATA_DECODER_PUBLIC_MOJOM_BUNDLED_EXCHANGES_PARSER_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/data_decoder/public/mojom/bundled_exchanges_parser.mojom-shared-internal.h"
#include "mojo/public/mojom/base/file.mojom-shared.h"
#include "url/mojom/url.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace data_decoder {
namespace mojom {
class BundleMetadataDataView;

class BundleIndexItemDataView;

class BundleResponseDataView;



}  // namespace mojom
}  // namespace data_decoder

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::data_decoder::mojom::BundleMetadataDataView> {
  using Data = ::data_decoder::mojom::internal::BundleMetadata_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::data_decoder::mojom::BundleIndexItemDataView> {
  using Data = ::data_decoder::mojom::internal::BundleIndexItem_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::data_decoder::mojom::BundleResponseDataView> {
  using Data = ::data_decoder::mojom::internal::BundleResponse_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace data_decoder {
namespace mojom {
// Interface base classes. They are used for type safety check.
class BundledExchangesParserFactoryInterfaceBase {};

using BundledExchangesParserFactoryPtrDataView =
    mojo::InterfacePtrDataView<BundledExchangesParserFactoryInterfaceBase>;
using BundledExchangesParserFactoryRequestDataView =
    mojo::InterfaceRequestDataView<BundledExchangesParserFactoryInterfaceBase>;
using BundledExchangesParserFactoryAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<BundledExchangesParserFactoryInterfaceBase>;
using BundledExchangesParserFactoryAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<BundledExchangesParserFactoryInterfaceBase>;
class BundledExchangesParserInterfaceBase {};

using BundledExchangesParserPtrDataView =
    mojo::InterfacePtrDataView<BundledExchangesParserInterfaceBase>;
using BundledExchangesParserRequestDataView =
    mojo::InterfaceRequestDataView<BundledExchangesParserInterfaceBase>;
using BundledExchangesParserAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<BundledExchangesParserInterfaceBase>;
using BundledExchangesParserAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<BundledExchangesParserInterfaceBase>;
class BundleDataSourceInterfaceBase {};

using BundleDataSourcePtrDataView =
    mojo::InterfacePtrDataView<BundleDataSourceInterfaceBase>;
using BundleDataSourceRequestDataView =
    mojo::InterfaceRequestDataView<BundleDataSourceInterfaceBase>;
using BundleDataSourceAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<BundleDataSourceInterfaceBase>;
using BundleDataSourceAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<BundleDataSourceInterfaceBase>;
class BundleMetadataDataView {
 public:
  BundleMetadataDataView() {}

  BundleMetadataDataView(
      internal::BundleMetadata_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetIndexDataView(
      mojo::ArrayDataView<BundleIndexItemDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIndex(UserType* output) {
    auto* pointer = data_->index.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::data_decoder::mojom::BundleIndexItemDataView>>(
        pointer, output, context_);
  }
  inline void GetManifestUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadManifestUrl(UserType* output) {
    auto* pointer = data_->manifest_url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::BundleMetadata_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BundleIndexItemDataView {
 public:
  BundleIndexItemDataView() {}

  BundleIndexItemDataView(
      internal::BundleIndexItem_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetRequestUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRequestUrl(UserType* output) {
    auto* pointer = data_->request_url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetRequestMethodDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRequestMethod(UserType* output) {
    auto* pointer = data_->request_method.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetRequestHeadersDataView(
      mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRequestHeaders(UserType* output) {
    auto* pointer = data_->request_headers.Get();
    return mojo::internal::Deserialize<mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>>(
        pointer, output, context_);
  }
  uint64_t response_offset() const {
    return data_->response_offset;
  }
  uint64_t response_length() const {
    return data_->response_length;
  }
 private:
  internal::BundleIndexItem_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BundleResponseDataView {
 public:
  BundleResponseDataView() {}

  BundleResponseDataView(
      internal::BundleResponse_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t response_code() const {
    return data_->response_code;
  }
  inline void GetResponseHeadersDataView(
      mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResponseHeaders(UserType* output) {
    auto* pointer = data_->response_headers.Get();
    return mojo::internal::Deserialize<mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>>(
        pointer, output, context_);
  }
  uint64_t payload_offset() const {
    return data_->payload_offset;
  }
  uint64_t payload_length() const {
    return data_->payload_length;
  }
 private:
  internal::BundleResponse_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace data_decoder

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::data_decoder::mojom::BundleMetadataDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::data_decoder::mojom::BundleMetadataDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::data_decoder::mojom::internal::BundleMetadata_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    decltype(Traits::index(input)) in_index = Traits::index(input);
    typename decltype((*output)->index)::BaseType::BufferWriter
        index_writer;
    const mojo::internal::ContainerValidateParams index_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::data_decoder::mojom::BundleIndexItemDataView>>(
        in_index, buffer, &index_writer, &index_validate_params,
        context);
    (*output)->index.Set(
        index_writer.is_null() ? nullptr : index_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->index.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null index in BundleMetadata struct");
    decltype(Traits::manifest_url(input)) in_manifest_url = Traits::manifest_url(input);
    typename decltype((*output)->manifest_url)::BaseType::BufferWriter
        manifest_url_writer;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_manifest_url, buffer, &manifest_url_writer, context);
    (*output)->manifest_url.Set(
        manifest_url_writer.is_null() ? nullptr : manifest_url_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->manifest_url.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null manifest_url in BundleMetadata struct");
  }

  static bool Deserialize(::data_decoder::mojom::internal::BundleMetadata_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::data_decoder::mojom::BundleMetadataDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::data_decoder::mojom::BundleIndexItemDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::data_decoder::mojom::BundleIndexItemDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::data_decoder::mojom::internal::BundleIndexItem_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    decltype(Traits::request_url(input)) in_request_url = Traits::request_url(input);
    typename decltype((*output)->request_url)::BaseType::BufferWriter
        request_url_writer;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_request_url, buffer, &request_url_writer, context);
    (*output)->request_url.Set(
        request_url_writer.is_null() ? nullptr : request_url_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->request_url.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null request_url in BundleIndexItem struct");
    decltype(Traits::request_method(input)) in_request_method = Traits::request_method(input);
    typename decltype((*output)->request_method)::BaseType::BufferWriter
        request_method_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_request_method, buffer, &request_method_writer, context);
    (*output)->request_method.Set(
        request_method_writer.is_null() ? nullptr : request_method_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->request_method.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null request_method in BundleIndexItem struct");
    decltype(Traits::request_headers(input)) in_request_headers = Traits::request_headers(input);
    typename decltype((*output)->request_headers)::BaseType::BufferWriter
        request_headers_writer;
    const mojo::internal::ContainerValidateParams request_headers_validate_params(
        new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)), new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)));
    mojo::internal::Serialize<mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>>(
        in_request_headers, buffer, &request_headers_writer, &request_headers_validate_params,
        context);
    (*output)->request_headers.Set(
        request_headers_writer.is_null() ? nullptr : request_headers_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->request_headers.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null request_headers in BundleIndexItem struct");
    (*output)->response_offset = Traits::response_offset(input);
    (*output)->response_length = Traits::response_length(input);
  }

  static bool Deserialize(::data_decoder::mojom::internal::BundleIndexItem_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::data_decoder::mojom::BundleIndexItemDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::data_decoder::mojom::BundleResponseDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::data_decoder::mojom::BundleResponseDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::data_decoder::mojom::internal::BundleResponse_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    (*output)->response_code = Traits::response_code(input);
    decltype(Traits::response_headers(input)) in_response_headers = Traits::response_headers(input);
    typename decltype((*output)->response_headers)::BaseType::BufferWriter
        response_headers_writer;
    const mojo::internal::ContainerValidateParams response_headers_validate_params(
        new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)), new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)));
    mojo::internal::Serialize<mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>>(
        in_response_headers, buffer, &response_headers_writer, &response_headers_validate_params,
        context);
    (*output)->response_headers.Set(
        response_headers_writer.is_null() ? nullptr : response_headers_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->response_headers.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null response_headers in BundleResponse struct");
    (*output)->payload_offset = Traits::payload_offset(input);
    (*output)->payload_length = Traits::payload_length(input);
  }

  static bool Deserialize(::data_decoder::mojom::internal::BundleResponse_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::data_decoder::mojom::BundleResponseDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace data_decoder {
namespace mojom {

inline void BundleMetadataDataView::GetIndexDataView(
    mojo::ArrayDataView<BundleIndexItemDataView>* output) {
  auto pointer = data_->index.Get();
  *output = mojo::ArrayDataView<BundleIndexItemDataView>(pointer, context_);
}
inline void BundleMetadataDataView::GetManifestUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->manifest_url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void BundleIndexItemDataView::GetRequestUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->request_url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void BundleIndexItemDataView::GetRequestMethodDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->request_method.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void BundleIndexItemDataView::GetRequestHeadersDataView(
    mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>* output) {
  auto pointer = data_->request_headers.Get();
  *output = mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>(pointer, context_);
}


inline void BundleResponseDataView::GetResponseHeadersDataView(
    mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>* output) {
  auto pointer = data_->response_headers.Get();
  *output = mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>(pointer, context_);
}



}  // namespace mojom
}  // namespace data_decoder

#endif  // SERVICES_DATA_DECODER_PUBLIC_MOJOM_BUNDLED_EXCHANGES_PARSER_MOJOM_SHARED_H_