// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_DATA_DECODER_PUBLIC_MOJOM_BUNDLED_EXCHANGES_PARSER_MOJOM_H_
#define SERVICES_DATA_DECODER_PUBLIC_MOJOM_BUNDLED_EXCHANGES_PARSER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/data_decoder/public/mojom/bundled_exchanges_parser.mojom-shared.h"
#include "services/data_decoder/public/mojom/bundled_exchanges_parser.mojom-forward.h"
#include "mojo/public/mojom/base/file.mojom.h"
#include "url/mojom/url.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace data_decoder {
namespace mojom {

class BundledExchangesParserFactoryProxy;

template <typename ImplRefTraits>
class BundledExchangesParserFactoryStub;

class BundledExchangesParserFactoryRequestValidator;


class  BundledExchangesParserFactory
    : public BundledExchangesParserFactoryInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Base_ = BundledExchangesParserFactoryInterfaceBase;
  using Proxy_ = BundledExchangesParserFactoryProxy;

  template <typename ImplRefTraits>
  using Stub_ = BundledExchangesParserFactoryStub<ImplRefTraits>;

  using RequestValidator_ = BundledExchangesParserFactoryRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kGetParserForFileMinVersion = 0,
    kGetParserForDataSourceMinVersion = 0,
  };
  virtual ~BundledExchangesParserFactory() {}

  
  virtual void GetParserForFile(mojo::PendingReceiver<BundledExchangesParser> receiver, base::File file) = 0;

  
  virtual void GetParserForDataSource(mojo::PendingReceiver<BundledExchangesParser> receiver, mojo::PendingRemote<BundleDataSource> data_source) = 0;
};

class BundledExchangesParserProxy;

template <typename ImplRefTraits>
class BundledExchangesParserStub;

class BundledExchangesParserRequestValidator;
class BundledExchangesParserResponseValidator;


class  BundledExchangesParser
    : public BundledExchangesParserInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Base_ = BundledExchangesParserInterfaceBase;
  using Proxy_ = BundledExchangesParserProxy;

  template <typename ImplRefTraits>
  using Stub_ = BundledExchangesParserStub<ImplRefTraits>;

  using RequestValidator_ = BundledExchangesParserRequestValidator;
  using ResponseValidator_ = BundledExchangesParserResponseValidator;
  enum MethodMinVersions : uint32_t {
    kParseMetadataMinVersion = 0,
    kParseResponseMinVersion = 0,
  };
  virtual ~BundledExchangesParser() {}


  using ParseMetadataCallback = base::OnceCallback<void(BundleMetadataPtr, const base::Optional<std::string>&)>;
  
  virtual void ParseMetadata(ParseMetadataCallback callback) = 0;


  using ParseResponseCallback = base::OnceCallback<void(BundleResponsePtr, const base::Optional<std::string>&)>;
  
  virtual void ParseResponse(uint64_t response_offset, uint64_t response_length, ParseResponseCallback callback) = 0;
};

class BundleDataSourceProxy;

template <typename ImplRefTraits>
class BundleDataSourceStub;

class BundleDataSourceRequestValidator;
class BundleDataSourceResponseValidator;


class  BundleDataSource
    : public BundleDataSourceInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Base_ = BundleDataSourceInterfaceBase;
  using Proxy_ = BundleDataSourceProxy;

  template <typename ImplRefTraits>
  using Stub_ = BundleDataSourceStub<ImplRefTraits>;

  using RequestValidator_ = BundleDataSourceRequestValidator;
  using ResponseValidator_ = BundleDataSourceResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGetSizeMinVersion = 0,
    kReadMinVersion = 0,
  };
  virtual ~BundleDataSource() {}


  using GetSizeCallback = base::OnceCallback<void(uint64_t)>;
  
  virtual void GetSize(GetSizeCallback callback) = 0;


  using ReadCallback = base::OnceCallback<void(const base::Optional<std::vector<uint8_t>>&)>;
  
  virtual void Read(uint64_t offset, uint64_t length, ReadCallback callback) = 0;
};

class  BundledExchangesParserFactoryProxy
    : public BundledExchangesParserFactory {
 public:
  using InterfaceType = BundledExchangesParserFactory;

  explicit BundledExchangesParserFactoryProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetParserForFile(mojo::PendingReceiver<BundledExchangesParser> receiver, base::File file) final;
  void GetParserForDataSource(mojo::PendingReceiver<BundledExchangesParser> receiver, mojo::PendingRemote<BundleDataSource> data_source) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  BundledExchangesParserProxy
    : public BundledExchangesParser {
 public:
  using InterfaceType = BundledExchangesParser;

  explicit BundledExchangesParserProxy(mojo::MessageReceiverWithResponder* receiver);
  void ParseMetadata(ParseMetadataCallback callback) final;
  void ParseResponse(uint64_t response_offset, uint64_t response_length, ParseResponseCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  BundleDataSourceProxy
    : public BundleDataSource {
 public:
  using InterfaceType = BundleDataSource;

  explicit BundleDataSourceProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetSize(GetSizeCallback callback) final;
  void Read(uint64_t offset, uint64_t length, ReadCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  BundledExchangesParserFactoryStubDispatch {
 public:
  static bool Accept(BundledExchangesParserFactory* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      BundledExchangesParserFactory* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<BundledExchangesParserFactory>>
class BundledExchangesParserFactoryStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  BundledExchangesParserFactoryStub() {}
  ~BundledExchangesParserFactoryStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BundledExchangesParserFactoryStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BundledExchangesParserFactoryStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  BundledExchangesParserStubDispatch {
 public:
  static bool Accept(BundledExchangesParser* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      BundledExchangesParser* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<BundledExchangesParser>>
class BundledExchangesParserStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  BundledExchangesParserStub() {}
  ~BundledExchangesParserStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BundledExchangesParserStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BundledExchangesParserStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  BundleDataSourceStubDispatch {
 public:
  static bool Accept(BundleDataSource* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      BundleDataSource* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<BundleDataSource>>
class BundleDataSourceStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  BundleDataSourceStub() {}
  ~BundleDataSourceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BundleDataSourceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BundleDataSourceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  BundledExchangesParserFactoryRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  BundledExchangesParserRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  BundleDataSourceRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  BundledExchangesParserResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  BundleDataSourceResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};










class  BundleMetadata {
 public:
  template <typename T>
  using EnableIfSame = std::enable_if_t<std::is_same<BundleMetadata, T>::value>;
  using DataView = BundleMetadataDataView;
  using Data_ = internal::BundleMetadata_Data;

  template <typename... Args>
  static BundleMetadataPtr New(Args&&... args) {
    return BundleMetadataPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static BundleMetadataPtr From(const U& u) {
    return mojo::TypeConverter<BundleMetadataPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, BundleMetadata>::Convert(*this);
  }


  BundleMetadata();

  BundleMetadata(
      std::vector<BundleIndexItemPtr> index,
      const GURL& manifest_url);

  ~BundleMetadata();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = BundleMetadataPtr>
  BundleMetadataPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T, BundleMetadata::EnableIfSame<T>* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        BundleMetadata::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        BundleMetadata::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::BundleMetadata_UnserializedMessageContext<
            UserType, BundleMetadata::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<BundleMetadata::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return BundleMetadata::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::BundleMetadata_UnserializedMessageContext<
            UserType, BundleMetadata::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<BundleMetadata::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  std::vector<BundleIndexItemPtr> index;
  
  GURL manifest_url;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(BundleMetadata);
};

// The comparison operators are templates, so they are only instantiated if they
// are used. Thus, the bindings generator does not need to know whether
// comparison operators are available for members.
template <typename T, BundleMetadata::EnableIfSame<T>* = nullptr>
bool operator<(const T& lhs, const T& rhs);

template <typename T, BundleMetadata::EnableIfSame<T>* = nullptr>
bool operator<=(const T& lhs, const T& rhs) {
  return !(rhs < lhs);
}

template <typename T, BundleMetadata::EnableIfSame<T>* = nullptr>
bool operator>(const T& lhs, const T& rhs) {
  return rhs < lhs;
}

template <typename T, BundleMetadata::EnableIfSame<T>* = nullptr>
bool operator>=(const T& lhs, const T& rhs) {
  return !(lhs < rhs);
}





class  BundleIndexItem {
 public:
  template <typename T>
  using EnableIfSame = std::enable_if_t<std::is_same<BundleIndexItem, T>::value>;
  using DataView = BundleIndexItemDataView;
  using Data_ = internal::BundleIndexItem_Data;

  template <typename... Args>
  static BundleIndexItemPtr New(Args&&... args) {
    return BundleIndexItemPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static BundleIndexItemPtr From(const U& u) {
    return mojo::TypeConverter<BundleIndexItemPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, BundleIndexItem>::Convert(*this);
  }


  BundleIndexItem();

  BundleIndexItem(
      const GURL& request_url,
      const std::string& request_method,
      const base::flat_map<std::string, std::string>& request_headers,
      uint64_t response_offset,
      uint64_t response_length);

  ~BundleIndexItem();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = BundleIndexItemPtr>
  BundleIndexItemPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T, BundleIndexItem::EnableIfSame<T>* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        BundleIndexItem::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        BundleIndexItem::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::BundleIndexItem_UnserializedMessageContext<
            UserType, BundleIndexItem::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<BundleIndexItem::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return BundleIndexItem::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::BundleIndexItem_UnserializedMessageContext<
            UserType, BundleIndexItem::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<BundleIndexItem::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  GURL request_url;
  
  std::string request_method;
  
  base::flat_map<std::string, std::string> request_headers;
  
  uint64_t response_offset;
  
  uint64_t response_length;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

// The comparison operators are templates, so they are only instantiated if they
// are used. Thus, the bindings generator does not need to know whether
// comparison operators are available for members.
template <typename T, BundleIndexItem::EnableIfSame<T>* = nullptr>
bool operator<(const T& lhs, const T& rhs);

template <typename T, BundleIndexItem::EnableIfSame<T>* = nullptr>
bool operator<=(const T& lhs, const T& rhs) {
  return !(rhs < lhs);
}

template <typename T, BundleIndexItem::EnableIfSame<T>* = nullptr>
bool operator>(const T& lhs, const T& rhs) {
  return rhs < lhs;
}

template <typename T, BundleIndexItem::EnableIfSame<T>* = nullptr>
bool operator>=(const T& lhs, const T& rhs) {
  return !(lhs < rhs);
}





class  BundleResponse {
 public:
  template <typename T>
  using EnableIfSame = std::enable_if_t<std::is_same<BundleResponse, T>::value>;
  using DataView = BundleResponseDataView;
  using Data_ = internal::BundleResponse_Data;

  template <typename... Args>
  static BundleResponsePtr New(Args&&... args) {
    return BundleResponsePtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static BundleResponsePtr From(const U& u) {
    return mojo::TypeConverter<BundleResponsePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, BundleResponse>::Convert(*this);
  }


  BundleResponse();

  BundleResponse(
      int32_t response_code,
      const base::flat_map<std::string, std::string>& response_headers,
      uint64_t payload_offset,
      uint64_t payload_length);

  ~BundleResponse();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = BundleResponsePtr>
  BundleResponsePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T, BundleResponse::EnableIfSame<T>* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        BundleResponse::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        BundleResponse::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::BundleResponse_UnserializedMessageContext<
            UserType, BundleResponse::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<BundleResponse::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return BundleResponse::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::BundleResponse_UnserializedMessageContext<
            UserType, BundleResponse::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<BundleResponse::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  int32_t response_code;
  
  base::flat_map<std::string, std::string> response_headers;
  
  uint64_t payload_offset;
  
  uint64_t payload_length;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

// The comparison operators are templates, so they are only instantiated if they
// are used. Thus, the bindings generator does not need to know whether
// comparison operators are available for members.
template <typename T, BundleResponse::EnableIfSame<T>* = nullptr>
bool operator<(const T& lhs, const T& rhs);

template <typename T, BundleResponse::EnableIfSame<T>* = nullptr>
bool operator<=(const T& lhs, const T& rhs) {
  return !(rhs < lhs);
}

template <typename T, BundleResponse::EnableIfSame<T>* = nullptr>
bool operator>(const T& lhs, const T& rhs) {
  return rhs < lhs;
}

template <typename T, BundleResponse::EnableIfSame<T>* = nullptr>
bool operator>=(const T& lhs, const T& rhs) {
  return !(lhs < rhs);
}

template <typename StructPtrType>
BundleMetadataPtr BundleMetadata::Clone() const {
  return New(
      mojo::Clone(index),
      mojo::Clone(manifest_url)
  );
}

template <typename T, BundleMetadata::EnableIfSame<T>*>
bool BundleMetadata::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->index, other_struct.index))
    return false;
  if (!mojo::Equals(this->manifest_url, other_struct.manifest_url))
    return false;
  return true;
}

template <typename T, BundleMetadata::EnableIfSame<T>*>
bool operator<(const T& lhs, const T& rhs) {
  if (std::less<decltype(lhs.index)>(lhs.index, rhs.index))
    return true;
  if (std::less<decltype(lhs.index)>(rhs.index, lhs.index))
    return false;
  if (std::less<decltype(lhs.manifest_url)>(lhs.manifest_url, rhs.manifest_url))
    return true;
  if (std::less<decltype(lhs.manifest_url)>(rhs.manifest_url, lhs.manifest_url))
    return false;
  return false;
}
template <typename StructPtrType>
BundleIndexItemPtr BundleIndexItem::Clone() const {
  return New(
      mojo::Clone(request_url),
      mojo::Clone(request_method),
      mojo::Clone(request_headers),
      mojo::Clone(response_offset),
      mojo::Clone(response_length)
  );
}

template <typename T, BundleIndexItem::EnableIfSame<T>*>
bool BundleIndexItem::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->request_url, other_struct.request_url))
    return false;
  if (!mojo::Equals(this->request_method, other_struct.request_method))
    return false;
  if (!mojo::Equals(this->request_headers, other_struct.request_headers))
    return false;
  if (!mojo::Equals(this->response_offset, other_struct.response_offset))
    return false;
  if (!mojo::Equals(this->response_length, other_struct.response_length))
    return false;
  return true;
}

template <typename T, BundleIndexItem::EnableIfSame<T>*>
bool operator<(const T& lhs, const T& rhs) {
  if (std::less<decltype(lhs.request_url)>(lhs.request_url, rhs.request_url))
    return true;
  if (std::less<decltype(lhs.request_url)>(rhs.request_url, lhs.request_url))
    return false;
  if (std::less<decltype(lhs.request_method)>(lhs.request_method, rhs.request_method))
    return true;
  if (std::less<decltype(lhs.request_method)>(rhs.request_method, lhs.request_method))
    return false;
  if (std::less<decltype(lhs.request_headers)>(lhs.request_headers, rhs.request_headers))
    return true;
  if (std::less<decltype(lhs.request_headers)>(rhs.request_headers, lhs.request_headers))
    return false;
  if (std::less<decltype(lhs.response_offset)>(lhs.response_offset, rhs.response_offset))
    return true;
  if (std::less<decltype(lhs.response_offset)>(rhs.response_offset, lhs.response_offset))
    return false;
  if (std::less<decltype(lhs.response_length)>(lhs.response_length, rhs.response_length))
    return true;
  if (std::less<decltype(lhs.response_length)>(rhs.response_length, lhs.response_length))
    return false;
  return false;
}
template <typename StructPtrType>
BundleResponsePtr BundleResponse::Clone() const {
  return New(
      mojo::Clone(response_code),
      mojo::Clone(response_headers),
      mojo::Clone(payload_offset),
      mojo::Clone(payload_length)
  );
}

template <typename T, BundleResponse::EnableIfSame<T>*>
bool BundleResponse::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->response_code, other_struct.response_code))
    return false;
  if (!mojo::Equals(this->response_headers, other_struct.response_headers))
    return false;
  if (!mojo::Equals(this->payload_offset, other_struct.payload_offset))
    return false;
  if (!mojo::Equals(this->payload_length, other_struct.payload_length))
    return false;
  return true;
}

template <typename T, BundleResponse::EnableIfSame<T>*>
bool operator<(const T& lhs, const T& rhs) {
  if (std::less<decltype(lhs.response_code)>(lhs.response_code, rhs.response_code))
    return true;
  if (std::less<decltype(lhs.response_code)>(rhs.response_code, lhs.response_code))
    return false;
  if (std::less<decltype(lhs.response_headers)>(lhs.response_headers, rhs.response_headers))
    return true;
  if (std::less<decltype(lhs.response_headers)>(rhs.response_headers, lhs.response_headers))
    return false;
  if (std::less<decltype(lhs.payload_offset)>(lhs.payload_offset, rhs.payload_offset))
    return true;
  if (std::less<decltype(lhs.payload_offset)>(rhs.payload_offset, lhs.payload_offset))
    return false;
  if (std::less<decltype(lhs.payload_length)>(lhs.payload_length, rhs.payload_length))
    return true;
  if (std::less<decltype(lhs.payload_length)>(rhs.payload_length, lhs.payload_length))
    return false;
  return false;
}


}  // namespace mojom
}  // namespace data_decoder

namespace mojo {


template <>
struct  StructTraits<::data_decoder::mojom::BundleMetadata::DataView,
                                         ::data_decoder::mojom::BundleMetadataPtr> {
  static bool IsNull(const ::data_decoder::mojom::BundleMetadataPtr& input) { return !input; }
  static void SetToNull(::data_decoder::mojom::BundleMetadataPtr* output) { output->reset(); }

  static const decltype(::data_decoder::mojom::BundleMetadata::index)& index(
      const ::data_decoder::mojom::BundleMetadataPtr& input) {
    return input->index;
  }

  static const decltype(::data_decoder::mojom::BundleMetadata::manifest_url)& manifest_url(
      const ::data_decoder::mojom::BundleMetadataPtr& input) {
    return input->manifest_url;
  }

  static bool Read(::data_decoder::mojom::BundleMetadata::DataView input, ::data_decoder::mojom::BundleMetadataPtr* output);
};


template <>
struct  StructTraits<::data_decoder::mojom::BundleIndexItem::DataView,
                                         ::data_decoder::mojom::BundleIndexItemPtr> {
  static bool IsNull(const ::data_decoder::mojom::BundleIndexItemPtr& input) { return !input; }
  static void SetToNull(::data_decoder::mojom::BundleIndexItemPtr* output) { output->reset(); }

  static const decltype(::data_decoder::mojom::BundleIndexItem::request_url)& request_url(
      const ::data_decoder::mojom::BundleIndexItemPtr& input) {
    return input->request_url;
  }

  static const decltype(::data_decoder::mojom::BundleIndexItem::request_method)& request_method(
      const ::data_decoder::mojom::BundleIndexItemPtr& input) {
    return input->request_method;
  }

  static const decltype(::data_decoder::mojom::BundleIndexItem::request_headers)& request_headers(
      const ::data_decoder::mojom::BundleIndexItemPtr& input) {
    return input->request_headers;
  }

  static decltype(::data_decoder::mojom::BundleIndexItem::response_offset) response_offset(
      const ::data_decoder::mojom::BundleIndexItemPtr& input) {
    return input->response_offset;
  }

  static decltype(::data_decoder::mojom::BundleIndexItem::response_length) response_length(
      const ::data_decoder::mojom::BundleIndexItemPtr& input) {
    return input->response_length;
  }

  static bool Read(::data_decoder::mojom::BundleIndexItem::DataView input, ::data_decoder::mojom::BundleIndexItemPtr* output);
};


template <>
struct  StructTraits<::data_decoder::mojom::BundleResponse::DataView,
                                         ::data_decoder::mojom::BundleResponsePtr> {
  static bool IsNull(const ::data_decoder::mojom::BundleResponsePtr& input) { return !input; }
  static void SetToNull(::data_decoder::mojom::BundleResponsePtr* output) { output->reset(); }

  static decltype(::data_decoder::mojom::BundleResponse::response_code) response_code(
      const ::data_decoder::mojom::BundleResponsePtr& input) {
    return input->response_code;
  }

  static const decltype(::data_decoder::mojom::BundleResponse::response_headers)& response_headers(
      const ::data_decoder::mojom::BundleResponsePtr& input) {
    return input->response_headers;
  }

  static decltype(::data_decoder::mojom::BundleResponse::payload_offset) payload_offset(
      const ::data_decoder::mojom::BundleResponsePtr& input) {
    return input->payload_offset;
  }

  static decltype(::data_decoder::mojom::BundleResponse::payload_length) payload_length(
      const ::data_decoder::mojom::BundleResponsePtr& input) {
    return input->payload_length;
  }

  static bool Read(::data_decoder::mojom::BundleResponse::DataView input, ::data_decoder::mojom::BundleResponsePtr* output);
};

}  // namespace mojo

#endif  // SERVICES_DATA_DECODER_PUBLIC_MOJOM_BUNDLED_EXCHANGES_PARSER_MOJOM_H_