// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_

#include <stddef.h>
#include "third_party/blink/renderer/core/core_export.h"
#include "third_party/blink/renderer/platform/wtf/allocator/allocator.h"
#include "third_party/blink/renderer/platform/wtf/assertions.h"

namespace WTF {
class AtomicString;
class String;
}

namespace blink {

enum class CSSPropertyID {
    kInvalid = 0,
    kVariable = 1,
    kAnimationDelay = 2,
    kAnimationDirection = 3,
    kAnimationDuration = 4,
    kAnimationFillMode = 5,
    kAnimationIterationCount = 6,
    kAnimationName = 7,
    kAnimationPlayState = 8,
    kAnimationTimingFunction = 9,
    kTransitionDelay = 10,
    kTransitionDuration = 11,
    kTransitionProperty = 12,
    kTransitionTimingFunction = 13,
    kColor = 14,
    kDirection = 15,
    kFontFamily = 16,
    kFontFeatureSettings = 17,
    kFontKerning = 18,
    kFontSize = 19,
    kFontSizeAdjust = 20,
    kFontStretch = 21,
    kFontStyle = 22,
    kFontVariantCaps = 23,
    kFontVariantEastAsian = 24,
    kFontVariantLigatures = 25,
    kFontVariantNumeric = 26,
    kFontVariationSettings = 27,
    kFontWeight = 28,
    kForcedColorAdjust = 29,
    kInternalEffectiveZoom = 30,
    kInternalVisitedColor = 31,
    kTextOrientation = 32,
    kTextRendering = 33,
    kWebkitFontSmoothing = 34,
    kWebkitLocale = 35,
    kWebkitTextOrientation = 36,
    kWebkitWritingMode = 37,
    kWritingMode = 38,
    kZoom = 39,
    kAlignContent = 40,
    kAlignItems = 41,
    kAlignSelf = 42,
    kAlignmentBaseline = 43,
    kAll = 44,
    kBackdropFilter = 45,
    kBackfaceVisibility = 46,
    kBackgroundAttachment = 47,
    kBackgroundBlendMode = 48,
    kBackgroundClip = 49,
    kBackgroundColor = 50,
    kBackgroundImage = 51,
    kBackgroundOrigin = 52,
    kBackgroundPositionX = 53,
    kBackgroundPositionY = 54,
    kBackgroundRepeatX = 55,
    kBackgroundRepeatY = 56,
    kBackgroundSize = 57,
    kBaselineShift = 58,
    kBlockSize = 59,
    kBorderBlockEndColor = 60,
    kBorderBlockEndStyle = 61,
    kBorderBlockEndWidth = 62,
    kBorderBlockStartColor = 63,
    kBorderBlockStartStyle = 64,
    kBorderBlockStartWidth = 65,
    kBorderBottomColor = 66,
    kBorderBottomLeftRadius = 67,
    kBorderBottomRightRadius = 68,
    kBorderBottomStyle = 69,
    kBorderBottomWidth = 70,
    kBorderCollapse = 71,
    kBorderImageOutset = 72,
    kBorderImageRepeat = 73,
    kBorderImageSlice = 74,
    kBorderImageSource = 75,
    kBorderImageWidth = 76,
    kBorderInlineEndColor = 77,
    kBorderInlineEndStyle = 78,
    kBorderInlineEndWidth = 79,
    kBorderInlineStartColor = 80,
    kBorderInlineStartStyle = 81,
    kBorderInlineStartWidth = 82,
    kBorderLeftColor = 83,
    kBorderLeftStyle = 84,
    kBorderLeftWidth = 85,
    kBorderRightColor = 86,
    kBorderRightStyle = 87,
    kBorderRightWidth = 88,
    kBorderTopColor = 89,
    kBorderTopLeftRadius = 90,
    kBorderTopRightRadius = 91,
    kBorderTopStyle = 92,
    kBorderTopWidth = 93,
    kBottom = 94,
    kBoxShadow = 95,
    kBoxSizing = 96,
    kBreakAfter = 97,
    kBreakBefore = 98,
    kBreakInside = 99,
    kBufferedRendering = 100,
    kCaptionSide = 101,
    kCaretColor = 102,
    kClear = 103,
    kClip = 104,
    kClipPath = 105,
    kClipRule = 106,
    kColorInterpolation = 107,
    kColorInterpolationFilters = 108,
    kColorRendering = 109,
    kColorScheme = 110,
    kColumnCount = 111,
    kColumnFill = 112,
    kColumnGap = 113,
    kColumnRuleColor = 114,
    kColumnRuleStyle = 115,
    kColumnRuleWidth = 116,
    kColumnSpan = 117,
    kColumnWidth = 118,
    kContain = 119,
    kContent = 120,
    kCounterIncrement = 121,
    kCounterReset = 122,
    kCursor = 123,
    kCx = 124,
    kCy = 125,
    kD = 126,
    kDisplay = 127,
    kDominantBaseline = 128,
    kEmptyCells = 129,
    kFill = 130,
    kFillOpacity = 131,
    kFillRule = 132,
    kFilter = 133,
    kFlexBasis = 134,
    kFlexDirection = 135,
    kFlexGrow = 136,
    kFlexShrink = 137,
    kFlexWrap = 138,
    kFloat = 139,
    kFloodColor = 140,
    kFloodOpacity = 141,
    kFontDisplay = 142,
    kGridAutoColumns = 143,
    kGridAutoFlow = 144,
    kGridAutoRows = 145,
    kGridColumnEnd = 146,
    kGridColumnStart = 147,
    kGridRowEnd = 148,
    kGridRowStart = 149,
    kGridTemplateAreas = 150,
    kGridTemplateColumns = 151,
    kGridTemplateRows = 152,
    kHeight = 153,
    kHyphens = 154,
    kImageOrientation = 155,
    kImageRendering = 156,
    kInherits = 157,
    kInitialValue = 158,
    kInlineSize = 159,
    kInsetBlockEnd = 160,
    kInsetBlockStart = 161,
    kInsetInlineEnd = 162,
    kInsetInlineStart = 163,
    kInternalVisitedBackgroundColor = 164,
    kInternalVisitedBorderBlockEndColor = 165,
    kInternalVisitedBorderBlockStartColor = 166,
    kInternalVisitedBorderBottomColor = 167,
    kInternalVisitedBorderInlineEndColor = 168,
    kInternalVisitedBorderInlineStartColor = 169,
    kInternalVisitedBorderLeftColor = 170,
    kInternalVisitedBorderRightColor = 171,
    kInternalVisitedBorderTopColor = 172,
    kInternalVisitedCaretColor = 173,
    kInternalVisitedColumnRuleColor = 174,
    kInternalVisitedFill = 175,
    kInternalVisitedOutlineColor = 176,
    kInternalVisitedStroke = 177,
    kInternalVisitedTextDecorationColor = 178,
    kInternalVisitedTextEmphasisColor = 179,
    kInternalVisitedTextFillColor = 180,
    kInternalVisitedTextStrokeColor = 181,
    kIsolation = 182,
    kJustifyContent = 183,
    kJustifyItems = 184,
    kJustifySelf = 185,
    kLeft = 186,
    kLetterSpacing = 187,
    kLightingColor = 188,
    kLineBreak = 189,
    kLineHeight = 190,
    kLineHeightStep = 191,
    kListStyleImage = 192,
    kListStylePosition = 193,
    kListStyleType = 194,
    kMarginBlockEnd = 195,
    kMarginBlockStart = 196,
    kMarginBottom = 197,
    kMarginInlineEnd = 198,
    kMarginInlineStart = 199,
    kMarginLeft = 200,
    kMarginRight = 201,
    kMarginTop = 202,
    kMarkerEnd = 203,
    kMarkerMid = 204,
    kMarkerStart = 205,
    kMask = 206,
    kMaskSourceType = 207,
    kMaskType = 208,
    kMaxBlockSize = 209,
    kMaxHeight = 210,
    kMaxInlineSize = 211,
    kMaxWidth = 212,
    kMaxZoom = 213,
    kMinBlockSize = 214,
    kMinHeight = 215,
    kMinInlineSize = 216,
    kMinWidth = 217,
    kMinZoom = 218,
    kMixBlendMode = 219,
    kObjectFit = 220,
    kObjectPosition = 221,
    kOffsetAnchor = 222,
    kOffsetDistance = 223,
    kOffsetPath = 224,
    kOffsetPosition = 225,
    kOffsetRotate = 226,
    kOpacity = 227,
    kOrder = 228,
    kOrientation = 229,
    kOrphans = 230,
    kOutlineColor = 231,
    kOutlineOffset = 232,
    kOutlineStyle = 233,
    kOutlineWidth = 234,
    kOverflowAnchor = 235,
    kOverflowBlock = 236,
    kOverflowInline = 237,
    kOverflowWrap = 238,
    kOverflowX = 239,
    kOverflowY = 240,
    kOverscrollBehaviorBlock = 241,
    kOverscrollBehaviorInline = 242,
    kOverscrollBehaviorX = 243,
    kOverscrollBehaviorY = 244,
    kPaddingBlockEnd = 245,
    kPaddingBlockStart = 246,
    kPaddingBottom = 247,
    kPaddingInlineEnd = 248,
    kPaddingInlineStart = 249,
    kPaddingLeft = 250,
    kPaddingRight = 251,
    kPaddingTop = 252,
    kPage = 253,
    kPaintOrder = 254,
    kPerspective = 255,
    kPerspectiveOrigin = 256,
    kPointerEvents = 257,
    kPosition = 258,
    kQuotes = 259,
    kR = 260,
    kResize = 261,
    kRight = 262,
    kRotate = 263,
    kRowGap = 264,
    kRx = 265,
    kRy = 266,
    kScale = 267,
    kScrollBehavior = 268,
    kScrollCustomization = 269,
    kScrollMarginBlockEnd = 270,
    kScrollMarginBlockStart = 271,
    kScrollMarginBottom = 272,
    kScrollMarginInlineEnd = 273,
    kScrollMarginInlineStart = 274,
    kScrollMarginLeft = 275,
    kScrollMarginRight = 276,
    kScrollMarginTop = 277,
    kScrollPaddingBlockEnd = 278,
    kScrollPaddingBlockStart = 279,
    kScrollPaddingBottom = 280,
    kScrollPaddingInlineEnd = 281,
    kScrollPaddingInlineStart = 282,
    kScrollPaddingLeft = 283,
    kScrollPaddingRight = 284,
    kScrollPaddingTop = 285,
    kScrollSnapAlign = 286,
    kScrollSnapStop = 287,
    kScrollSnapType = 288,
    kShapeImageThreshold = 289,
    kShapeMargin = 290,
    kShapeOutside = 291,
    kShapeRendering = 292,
    kSize = 293,
    kSpeak = 294,
    kSrc = 295,
    kStopColor = 296,
    kStopOpacity = 297,
    kStroke = 298,
    kStrokeDasharray = 299,
    kStrokeDashoffset = 300,
    kStrokeLinecap = 301,
    kStrokeLinejoin = 302,
    kStrokeMiterlimit = 303,
    kStrokeOpacity = 304,
    kStrokeWidth = 305,
    kSyntax = 306,
    kTabSize = 307,
    kTableLayout = 308,
    kTextAlign = 309,
    kTextAlignLast = 310,
    kTextAnchor = 311,
    kTextCombineUpright = 312,
    kTextDecorationColor = 313,
    kTextDecorationLine = 314,
    kTextDecorationSkipInk = 315,
    kTextDecorationStyle = 316,
    kTextIndent = 317,
    kTextJustify = 318,
    kTextOverflow = 319,
    kTextShadow = 320,
    kTextSizeAdjust = 321,
    kTextTransform = 322,
    kTextUnderlinePosition = 323,
    kTop = 324,
    kTouchAction = 325,
    kTransform = 326,
    kTransformBox = 327,
    kTransformOrigin = 328,
    kTransformStyle = 329,
    kTranslate = 330,
    kUnicodeBidi = 331,
    kUnicodeRange = 332,
    kUserSelect = 333,
    kUserZoom = 334,
    kVectorEffect = 335,
    kVerticalAlign = 336,
    kViewportFit = 337,
    kVisibility = 338,
    kWebkitAppRegion = 339,
    kWebkitAppearance = 340,
    kWebkitBorderHorizontalSpacing = 341,
    kWebkitBorderImage = 342,
    kWebkitBorderVerticalSpacing = 343,
    kWebkitBoxAlign = 344,
    kWebkitBoxDecorationBreak = 345,
    kWebkitBoxDirection = 346,
    kWebkitBoxFlex = 347,
    kWebkitBoxOrdinalGroup = 348,
    kWebkitBoxOrient = 349,
    kWebkitBoxPack = 350,
    kWebkitBoxReflect = 351,
    kWebkitFontSizeDelta = 352,
    kWebkitHighlight = 353,
    kWebkitHyphenateCharacter = 354,
    kWebkitLineBreak = 355,
    kWebkitLineClamp = 356,
    kWebkitMarginAfterCollapse = 357,
    kWebkitMarginBeforeCollapse = 358,
    kWebkitMarginBottomCollapse = 359,
    kWebkitMarginTopCollapse = 360,
    kWebkitMaskBoxImageOutset = 361,
    kWebkitMaskBoxImageRepeat = 362,
    kWebkitMaskBoxImageSlice = 363,
    kWebkitMaskBoxImageSource = 364,
    kWebkitMaskBoxImageWidth = 365,
    kWebkitMaskClip = 366,
    kWebkitMaskComposite = 367,
    kWebkitMaskImage = 368,
    kWebkitMaskOrigin = 369,
    kWebkitMaskPositionX = 370,
    kWebkitMaskPositionY = 371,
    kWebkitMaskRepeatX = 372,
    kWebkitMaskRepeatY = 373,
    kWebkitMaskSize = 374,
    kWebkitPerspectiveOriginX = 375,
    kWebkitPerspectiveOriginY = 376,
    kWebkitPrintColorAdjust = 377,
    kWebkitRtlOrdering = 378,
    kWebkitRubyPosition = 379,
    kWebkitTapHighlightColor = 380,
    kWebkitTextCombine = 381,
    kWebkitTextDecorationsInEffect = 382,
    kWebkitTextEmphasisColor = 383,
    kWebkitTextEmphasisPosition = 384,
    kWebkitTextEmphasisStyle = 385,
    kWebkitTextFillColor = 386,
    kWebkitTextSecurity = 387,
    kWebkitTextStrokeColor = 388,
    kWebkitTextStrokeWidth = 389,
    kWebkitTransformOriginX = 390,
    kWebkitTransformOriginY = 391,
    kWebkitTransformOriginZ = 392,
    kWebkitUserDrag = 393,
    kWebkitUserModify = 394,
    kWhiteSpace = 395,
    kWidows = 396,
    kWidth = 397,
    kWillChange = 398,
    kWordBreak = 399,
    kWordSpacing = 400,
    kX = 401,
    kY = 402,
    kZIndex = 403,
    kAnimation = 404,
    kBackground = 405,
    kBackgroundPosition = 406,
    kBackgroundRepeat = 407,
    kBorder = 408,
    kBorderBlock = 409,
    kBorderBlockColor = 410,
    kBorderBlockEnd = 411,
    kBorderBlockStart = 412,
    kBorderBlockStyle = 413,
    kBorderBlockWidth = 414,
    kBorderBottom = 415,
    kBorderColor = 416,
    kBorderImage = 417,
    kBorderInline = 418,
    kBorderInlineColor = 419,
    kBorderInlineEnd = 420,
    kBorderInlineStart = 421,
    kBorderInlineStyle = 422,
    kBorderInlineWidth = 423,
    kBorderLeft = 424,
    kBorderRadius = 425,
    kBorderRight = 426,
    kBorderSpacing = 427,
    kBorderStyle = 428,
    kBorderTop = 429,
    kBorderWidth = 430,
    kColumnRule = 431,
    kColumns = 432,
    kFlex = 433,
    kFlexFlow = 434,
    kFont = 435,
    kFontVariant = 436,
    kGap = 437,
    kGrid = 438,
    kGridArea = 439,
    kGridColumn = 440,
    kGridColumnGap = 441,
    kGridGap = 442,
    kGridRow = 443,
    kGridRowGap = 444,
    kGridTemplate = 445,
    kInset = 446,
    kInsetBlock = 447,
    kInsetInline = 448,
    kListStyle = 449,
    kMargin = 450,
    kMarginBlock = 451,
    kMarginInline = 452,
    kMarker = 453,
    kOffset = 454,
    kOutline = 455,
    kOverflow = 456,
    kOverscrollBehavior = 457,
    kPadding = 458,
    kPaddingBlock = 459,
    kPaddingInline = 460,
    kPageBreakAfter = 461,
    kPageBreakBefore = 462,
    kPageBreakInside = 463,
    kPlaceContent = 464,
    kPlaceItems = 465,
    kPlaceSelf = 466,
    kScrollMargin = 467,
    kScrollMarginBlock = 468,
    kScrollMarginInline = 469,
    kScrollPadding = 470,
    kScrollPaddingBlock = 471,
    kScrollPaddingInline = 472,
    kTextDecoration = 473,
    kTransition = 474,
    kWebkitColumnBreakAfter = 475,
    kWebkitColumnBreakBefore = 476,
    kWebkitColumnBreakInside = 477,
    kWebkitMarginCollapse = 478,
    kWebkitMask = 479,
    kWebkitMaskBoxImage = 480,
    kWebkitMaskPosition = 481,
    kWebkitMaskRepeat = 482,
    kWebkitTextEmphasis = 483,
    kWebkitTextStroke = 484,
    kAliasWebkitBorderEndColor = 589,
    kAliasWebkitBorderEndStyle = 590,
    kAliasWebkitBorderEndWidth = 591,
    kAliasWebkitBorderStartColor = 592,
    kAliasWebkitBorderStartStyle = 593,
    kAliasWebkitBorderStartWidth = 594,
    kAliasWebkitBorderBeforeColor = 575,
    kAliasWebkitBorderBeforeStyle = 576,
    kAliasWebkitBorderBeforeWidth = 577,
    kAliasWebkitBorderAfterColor = 572,
    kAliasWebkitBorderAfterStyle = 573,
    kAliasWebkitBorderAfterWidth = 574,
    kAliasWebkitMarginEnd = 710,
    kAliasWebkitMarginStart = 711,
    kAliasWebkitMarginBefore = 708,
    kAliasWebkitMarginAfter = 707,
    kAliasWebkitPaddingEnd = 760,
    kAliasWebkitPaddingStart = 761,
    kAliasWebkitPaddingBefore = 758,
    kAliasWebkitPaddingAfter = 757,
    kAliasWebkitLogicalWidth = 671,
    kAliasWebkitLogicalHeight = 571,
    kAliasWebkitMinLogicalWidth = 728,
    kAliasWebkitMinLogicalHeight = 726,
    kAliasWebkitMaxLogicalWidth = 723,
    kAliasWebkitMaxLogicalHeight = 721,
    kAliasWebkitBorderAfter = 923,
    kAliasWebkitBorderBefore = 924,
    kAliasWebkitBorderEnd = 932,
    kAliasWebkitBorderStart = 933,
    kAliasEpubCaptionSide = 613,
    kAliasEpubTextCombine = 893,
    kAliasEpubTextEmphasis = 995,
    kAliasEpubTextEmphasisColor = 895,
    kAliasEpubTextEmphasisStyle = 897,
    kAliasEpubTextOrientation = 548,
    kAliasEpubTextTransform = 834,
    kAliasEpubWordBreak = 911,
    kAliasEpubWritingMode = 549,
    kAliasWebkitAlignContent = 552,
    kAliasWebkitAlignItems = 553,
    kAliasWebkitAlignSelf = 554,
    kAliasWebkitAnimation = 916,
    kAliasWebkitAnimationDelay = 514,
    kAliasWebkitAnimationDirection = 515,
    kAliasWebkitAnimationDuration = 516,
    kAliasWebkitAnimationFillMode = 517,
    kAliasWebkitAnimationIterationCount = 518,
    kAliasWebkitAnimationName = 519,
    kAliasWebkitAnimationPlayState = 520,
    kAliasWebkitAnimationTimingFunction = 521,
    kAliasWebkitBackfaceVisibility = 558,
    kAliasWebkitBackgroundClip = 561,
    kAliasWebkitBackgroundOrigin = 564,
    kAliasWebkitBackgroundSize = 569,
    kAliasWebkitBorderBottomLeftRadius = 579,
    kAliasWebkitBorderBottomRightRadius = 580,
    kAliasWebkitBorderRadius = 937,
    kAliasWebkitBorderTopLeftRadius = 602,
    kAliasWebkitBorderTopRightRadius = 603,
    kAliasWebkitBoxShadow = 607,
    kAliasWebkitBoxSizing = 608,
    kAliasWebkitClipPath = 617,
    kAliasWebkitColumnCount = 623,
    kAliasWebkitColumnGap = 625,
    kAliasWebkitColumnRule = 943,
    kAliasWebkitColumnRuleColor = 626,
    kAliasWebkitColumnRuleStyle = 627,
    kAliasWebkitColumnRuleWidth = 628,
    kAliasWebkitColumnSpan = 629,
    kAliasWebkitColumnWidth = 630,
    kAliasWebkitColumns = 944,
    kAliasWebkitFilter = 645,
    kAliasWebkitFlex = 945,
    kAliasWebkitFlexBasis = 646,
    kAliasWebkitFlexDirection = 647,
    kAliasWebkitFlexFlow = 946,
    kAliasWebkitFlexGrow = 648,
    kAliasWebkitFlexShrink = 649,
    kAliasWebkitFlexWrap = 650,
    kAliasWebkitFontFeatureSettings = 529,
    kAliasWebkitJustifyContent = 695,
    kAliasWebkitOpacity = 739,
    kAliasWebkitOrder = 740,
    kAliasWebkitPerspective = 767,
    kAliasWebkitPerspectiveOrigin = 768,
    kAliasWebkitShapeImageThreshold = 801,
    kAliasWebkitShapeMargin = 802,
    kAliasWebkitShapeOutside = 803,
    kAliasWebkitTextSizeAdjust = 833,
    kAliasWebkitTransform = 838,
    kAliasWebkitTransformOrigin = 840,
    kAliasWebkitTransformStyle = 841,
    kAliasWebkitTransition = 986,
    kAliasWebkitTransitionDelay = 522,
    kAliasWebkitTransitionDuration = 523,
    kAliasWebkitTransitionProperty = 524,
    kAliasWebkitTransitionTimingFunction = 525,
    kAliasWebkitUserSelect = 845,
    kAliasWordWrap = 750,
};

const CSSPropertyID kCSSPropertyAliasList[] = {
    CSSPropertyID::kAliasWebkitBorderEndColor,
    CSSPropertyID::kAliasWebkitBorderEndStyle,
    CSSPropertyID::kAliasWebkitBorderEndWidth,
    CSSPropertyID::kAliasWebkitBorderStartColor,
    CSSPropertyID::kAliasWebkitBorderStartStyle,
    CSSPropertyID::kAliasWebkitBorderStartWidth,
    CSSPropertyID::kAliasWebkitBorderBeforeColor,
    CSSPropertyID::kAliasWebkitBorderBeforeStyle,
    CSSPropertyID::kAliasWebkitBorderBeforeWidth,
    CSSPropertyID::kAliasWebkitBorderAfterColor,
    CSSPropertyID::kAliasWebkitBorderAfterStyle,
    CSSPropertyID::kAliasWebkitBorderAfterWidth,
    CSSPropertyID::kAliasWebkitMarginEnd,
    CSSPropertyID::kAliasWebkitMarginStart,
    CSSPropertyID::kAliasWebkitMarginBefore,
    CSSPropertyID::kAliasWebkitMarginAfter,
    CSSPropertyID::kAliasWebkitPaddingEnd,
    CSSPropertyID::kAliasWebkitPaddingStart,
    CSSPropertyID::kAliasWebkitPaddingBefore,
    CSSPropertyID::kAliasWebkitPaddingAfter,
    CSSPropertyID::kAliasWebkitLogicalWidth,
    CSSPropertyID::kAliasWebkitLogicalHeight,
    CSSPropertyID::kAliasWebkitMinLogicalWidth,
    CSSPropertyID::kAliasWebkitMinLogicalHeight,
    CSSPropertyID::kAliasWebkitMaxLogicalWidth,
    CSSPropertyID::kAliasWebkitMaxLogicalHeight,
    CSSPropertyID::kAliasWebkitBorderAfter,
    CSSPropertyID::kAliasWebkitBorderBefore,
    CSSPropertyID::kAliasWebkitBorderEnd,
    CSSPropertyID::kAliasWebkitBorderStart,
    CSSPropertyID::kAliasEpubCaptionSide,
    CSSPropertyID::kAliasEpubTextCombine,
    CSSPropertyID::kAliasEpubTextEmphasis,
    CSSPropertyID::kAliasEpubTextEmphasisColor,
    CSSPropertyID::kAliasEpubTextEmphasisStyle,
    CSSPropertyID::kAliasEpubTextOrientation,
    CSSPropertyID::kAliasEpubTextTransform,
    CSSPropertyID::kAliasEpubWordBreak,
    CSSPropertyID::kAliasEpubWritingMode,
    CSSPropertyID::kAliasWebkitAlignContent,
    CSSPropertyID::kAliasWebkitAlignItems,
    CSSPropertyID::kAliasWebkitAlignSelf,
    CSSPropertyID::kAliasWebkitAnimation,
    CSSPropertyID::kAliasWebkitAnimationDelay,
    CSSPropertyID::kAliasWebkitAnimationDirection,
    CSSPropertyID::kAliasWebkitAnimationDuration,
    CSSPropertyID::kAliasWebkitAnimationFillMode,
    CSSPropertyID::kAliasWebkitAnimationIterationCount,
    CSSPropertyID::kAliasWebkitAnimationName,
    CSSPropertyID::kAliasWebkitAnimationPlayState,
    CSSPropertyID::kAliasWebkitAnimationTimingFunction,
    CSSPropertyID::kAliasWebkitBackfaceVisibility,
    CSSPropertyID::kAliasWebkitBackgroundClip,
    CSSPropertyID::kAliasWebkitBackgroundOrigin,
    CSSPropertyID::kAliasWebkitBackgroundSize,
    CSSPropertyID::kAliasWebkitBorderBottomLeftRadius,
    CSSPropertyID::kAliasWebkitBorderBottomRightRadius,
    CSSPropertyID::kAliasWebkitBorderRadius,
    CSSPropertyID::kAliasWebkitBorderTopLeftRadius,
    CSSPropertyID::kAliasWebkitBorderTopRightRadius,
    CSSPropertyID::kAliasWebkitBoxShadow,
    CSSPropertyID::kAliasWebkitBoxSizing,
    CSSPropertyID::kAliasWebkitClipPath,
    CSSPropertyID::kAliasWebkitColumnCount,
    CSSPropertyID::kAliasWebkitColumnGap,
    CSSPropertyID::kAliasWebkitColumnRule,
    CSSPropertyID::kAliasWebkitColumnRuleColor,
    CSSPropertyID::kAliasWebkitColumnRuleStyle,
    CSSPropertyID::kAliasWebkitColumnRuleWidth,
    CSSPropertyID::kAliasWebkitColumnSpan,
    CSSPropertyID::kAliasWebkitColumnWidth,
    CSSPropertyID::kAliasWebkitColumns,
    CSSPropertyID::kAliasWebkitFilter,
    CSSPropertyID::kAliasWebkitFlex,
    CSSPropertyID::kAliasWebkitFlexBasis,
    CSSPropertyID::kAliasWebkitFlexDirection,
    CSSPropertyID::kAliasWebkitFlexFlow,
    CSSPropertyID::kAliasWebkitFlexGrow,
    CSSPropertyID::kAliasWebkitFlexShrink,
    CSSPropertyID::kAliasWebkitFlexWrap,
    CSSPropertyID::kAliasWebkitFontFeatureSettings,
    CSSPropertyID::kAliasWebkitJustifyContent,
    CSSPropertyID::kAliasWebkitOpacity,
    CSSPropertyID::kAliasWebkitOrder,
    CSSPropertyID::kAliasWebkitPerspective,
    CSSPropertyID::kAliasWebkitPerspectiveOrigin,
    CSSPropertyID::kAliasWebkitShapeImageThreshold,
    CSSPropertyID::kAliasWebkitShapeMargin,
    CSSPropertyID::kAliasWebkitShapeOutside,
    CSSPropertyID::kAliasWebkitTextSizeAdjust,
    CSSPropertyID::kAliasWebkitTransform,
    CSSPropertyID::kAliasWebkitTransformOrigin,
    CSSPropertyID::kAliasWebkitTransformStyle,
    CSSPropertyID::kAliasWebkitTransition,
    CSSPropertyID::kAliasWebkitTransitionDelay,
    CSSPropertyID::kAliasWebkitTransitionDuration,
    CSSPropertyID::kAliasWebkitTransitionProperty,
    CSSPropertyID::kAliasWebkitTransitionTimingFunction,
    CSSPropertyID::kAliasWebkitUserSelect,
    CSSPropertyID::kAliasWordWrap,
};

const int kIntFirstCSSProperty = 2;
const CSSPropertyID firstCSSProperty = static_cast<CSSPropertyID>(kIntFirstCSSProperty);
const int numCSSProperties = 583;
const int kIntLastCSSProperty = 484;
const CSSPropertyID lastCSSProperty = static_cast<CSSPropertyID>(kIntLastCSSProperty);
const CSSPropertyID lastUnresolvedCSSProperty = static_cast<CSSPropertyID>(995);
const int numCSSPropertyIDs = static_cast<int>(lastUnresolvedCSSProperty) + 1;
const size_t maxCSSPropertyNameLength = 48;

inline int GetCSSPropertyIDIndex(CSSPropertyID id) {
    DCHECK_GE(id, firstCSSProperty);
    DCHECK_LE(id, lastCSSProperty);
    return static_cast<int>(id) - kIntFirstCSSProperty;
}

inline bool isCSSPropertyIDWithName(CSSPropertyID id)
{
    return id >= firstCSSProperty && id <= lastUnresolvedCSSProperty;
}

inline bool isValidCSSPropertyID(CSSPropertyID id)
{
    return id != CSSPropertyID::kInvalid;
}

inline CSSPropertyID convertToCSSPropertyID(int value)
{
    DCHECK_GE(value, static_cast<int>(CSSPropertyID::kInvalid));
    DCHECK_LE(value, kIntLastCSSProperty);
    return static_cast<CSSPropertyID>(value);
}

inline CSSPropertyID resolveCSSPropertyID(CSSPropertyID id)
{
    return convertToCSSPropertyID(static_cast<int>(id) & ~512);
}

inline bool isPropertyAlias(CSSPropertyID id) { return static_cast<int>(id) & 512; }

CSSPropertyID CORE_EXPORT unresolvedCSSPropertyID(const WTF::String&);

CSSPropertyID CORE_EXPORT cssPropertyID(const WTF::String&);

class CSSPropertyIDList {
  STACK_ALLOCATED();
 public:
    class Iterator {
      STACK_ALLOCATED();
     public:
      Iterator(int id) : id_(id) {}
      CSSPropertyID operator*() const { return convertToCSSPropertyID(id_); }
      Iterator& operator++() { id_++; return *this; }
      bool operator!=(const Iterator& i) const { return id_ != i.id_; }
     private:
      int id_;
    };
  Iterator begin() const { return Iterator(kIntFirstCSSProperty); }
  Iterator end() const { return Iterator(kIntLastCSSProperty + 1); }
};

}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_
